'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _class, _class2, _temp2; /**
                              * @fileOverview Area Chart
                              */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _Surface = require('../container/Surface');

var _Surface2 = _interopRequireDefault(_Surface);

var _Tooltip = require('../component/Tooltip');

var _Tooltip2 = _interopRequireDefault(_Tooltip);

var _Dot = require('../shape/Dot');

var _Dot2 = _interopRequireDefault(_Dot);

var _Curve = require('../shape/Curve');

var _Curve2 = _interopRequireDefault(_Curve);

var _ReactUtils = require('../util/ReactUtils');

var _CartesianChart2 = require('./CartesianChart');

var _CartesianChart3 = _interopRequireDefault(_CartesianChart2);

var _Area = require('../cartesian/Area');

var _Area2 = _interopRequireDefault(_Area);

var _PureRender = require('../util/PureRender');

var _PureRender2 = _interopRequireDefault(_PureRender);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var AreaChart = (0, _PureRender2.default)(_class = (_temp2 = _class2 = function (_CartesianChart) {
  _inherits(AreaChart, _CartesianChart);

  function AreaChart() {
    var _Object$getPrototypeO;

    var _temp, _this, _ret;

    _classCallCheck(this, AreaChart);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = _possibleConstructorReturn(this, (_Object$getPrototypeO = Object.getPrototypeOf(AreaChart)).call.apply(_Object$getPrototypeO, [this].concat(args))), _this), _this.handleAreaMouseLeave = function () {
      _this.setState({
        activeAreaKey: null
      });
    }, _temp), _possibleConstructorReturn(_this, _ret);
  }

  _createClass(AreaChart, [{
    key: 'getComposedData',


    /**
     * Compose the data of each area
     * @param  {Object} xAxis       The configuration of x-axis
     * @param  {Object} yAxis       The configuration of y-axis
     * @param  {String} dataKey     The unique key of a group
     * @param  {Array}  stackedData If the area is stacked,
     * the stackedData is an array of min value and max value
     * @return {Array} Composed data
     */
    value: function getComposedData(xAxis, yAxis, dataKey, stackedData) {
      var _props = this.props;
      var data = _props.data;
      var layout = _props.layout;

      var xTicks = this.getAxisTicks(xAxis);
      var yTicks = this.getAxisTicks(yAxis);
      var hasStack = stackedData && stackedData.length;

      var points = data.map(function (entry, index) {
        var value = hasStack ? stackedData[index] : [0, entry[dataKey]];
        return {
          x: layout === 'horizontal' ? xTicks[index].coord : xAxis.scale(value[1]),
          y: layout === 'horizontal' ? yAxis.scale(value[1]) : yTicks[index].coord,
          value: entry[dataKey]
        };
      });

      var range = undefined;
      var baseLine = undefined;
      if (hasStack) {
        baseLine = stackedData.map(function (entry, index) {
          return {
            x: layout === 'horizontal' ? xTicks[index].coord : xAxis.scale(entry[0]),
            y: layout === 'horizontal' ? yAxis.scale(entry[0]) : yTicks[index].coord
          };
        });
      } else if (layout === 'horizontal') {
        range = yAxis.scale.range();
        baseLine = xAxis.orientation === 'top' ? Math.min(range[0], range[1]) : Math.max(range[0], range[1]);
      } else {
        range = xAxis.scale.range();
        baseLine = yAxis.orientation === 'left' ? Math.min(range[0], range[1]) : Math.max(range[0], range[1]);
      }

      return { points: points, baseLine: baseLine, layout: layout };
    }

    /**
     * Handler of mouse entering area chart
     * @param {String} key  The unique key of a group of data
     * @return {Object} null
     */

  }, {
    key: 'handleAreaMouseEnter',
    value: function handleAreaMouseEnter(key) {
      this.setState({
        activeAreaKey: key
      });
    }

    /**
     * Handler of mouse leaving area chart
     * @return {Object} null
     */

  }, {
    key: 'renderCursor',
    value: function renderCursor(xAxisMap, yAxisMap, offset) {
      var children = this.props.children;

      var tooltipItem = (0, _ReactUtils.findChildByType)(children, _Tooltip2.default);

      if (!tooltipItem || !tooltipItem.props.cursor || !this.state.isTooltipActive) {
        return null;
      }

      var layout = this.props.layout;
      var activeTooltipIndex = this.state.activeTooltipIndex;

      var axisMap = layout === 'horizontal' ? xAxisMap : yAxisMap;
      var ids = Object.keys(axisMap);
      var axis = axisMap[ids[0]];
      var ticks = this.getAxisTicks(axis);
      var start = ticks[activeTooltipIndex].coord;
      var x1 = layout === 'horizontal' ? start : offset.left;
      var y1 = layout === 'horizontal' ? offset.top : start;
      var x2 = layout === 'horizontal' ? start : offset.left + offset.width;
      var y2 = layout === 'horizontal' ? offset.top + offset.height : start;
      var cursorProps = _extends({
        stroke: '#ccc'
      }, (0, _ReactUtils.getPresentationAttributes)(tooltipItem.props.cursor), {
        points: [{ x: x1, y: y1 }, { x: x2, y: y2 }]
      });

      return _react2.default.isValidElement(tooltipItem.props.cursor) ? _react2.default.cloneElement(tooltipItem.props.cursor, cursorProps) : _react2.default.createElement(_Curve2.default, _extends({}, cursorProps, { type: 'linear', className: 'recharts-tooltip-cursor' }));
    }

    /**
     * Draw the main part of area chart
     * @param  {Array} items     React elements of Area
     * @param  {Object} xAxisMap The configuration of all x-axis
     * @param  {Object} yAxisMap The configuration of all y-axis
     * @param  {Object} offset   The offset of main part in the svg element
     * @param  {Object} stackGroups The items grouped by axisId and stackId
     * @return {ReactComponent} The instances of Area
     */

  }, {
    key: 'renderItems',
    value: function renderItems(items, xAxisMap, yAxisMap, offset, stackGroups) {
      var _this2 = this;

      var _props2 = this.props;
      var children = _props2.children;
      var layout = _props2.layout;
      var _state = this.state;
      var activeAreaKey = _state.activeAreaKey;
      var isTooltipActive = _state.isTooltipActive;
      var activeTooltipIndex = _state.activeTooltipIndex;

      var tooltipItem = (0, _ReactUtils.findChildByType)(children, _Tooltip2.default);
      var hasDot = tooltipItem && isTooltipActive;
      var dotItems = [];

      var areaItems = items.reduce(function (result, child, i) {
        var _child$props = child.props;
        var xAxisId = _child$props.xAxisId;
        var yAxisId = _child$props.yAxisId;
        var dataKey = _child$props.dataKey;
        var fillOpacity = _child$props.fillOpacity;
        var fill = _child$props.fill;

        var axisId = layout === 'horizontal' ? xAxisId : yAxisId;
        var stackedData = stackGroups && stackGroups[axisId] && stackGroups[axisId].hasStack && _this2.getStackedDataOfItem(child, stackGroups[axisId].stackGroups);
        var composeData = _this2.getComposedData(xAxisMap[xAxisId], yAxisMap[yAxisId], dataKey, stackedData);

        var activePoint = composeData.points && composeData.points[activeTooltipIndex];
        var pointStyle = { fill: fill, strokeWidth: 2, stroke: '#fff' };

        if (hasDot && activePoint) {
          dotItems.push(_react2.default.createElement(_Dot2.default, _extends({
            key: 'area-dot-' + i,
            cx: activePoint.x,
            cy: activePoint.y,
            r: 4
          }, pointStyle)));
        }

        var finalFillOpacity = fillOpacity === +fillOpacity ? fillOpacity : _Area2.default.defaultProps.fillOpacity;
        finalFillOpacity = activeAreaKey === dataKey ? Math.min(finalFillOpacity * 1.2, 1) : finalFillOpacity;

        var area = _react2.default.cloneElement(child, _extends({
          key: 'area-' + i
        }, offset, composeData, {
          fillOpacity: finalFillOpacity,
          onMouseLeave: _this2.handleAreaMouseLeave,
          onMouseEnter: _this2.handleAreaMouseEnter.bind(_this2, dataKey)
        }));

        return activeAreaKey === dataKey ? [].concat(_toConsumableArray(result), [area]) : [area].concat(_toConsumableArray(result));
      }, []);

      return _react2.default.createElement(
        'g',
        { key: 'recharts-area-wrapper' },
        _react2.default.createElement(
          'g',
          { key: 'recharts-area' },
          areaItems
        ),
        _react2.default.createElement(
          'g',
          { key: 'recharts-area-dot' },
          dotItems
        )
      );
    }
  }, {
    key: 'render',
    value: function render() {
      if (!(0, _ReactUtils.validateWidthHeight)(this)) {
        return null;
      }

      var _props3 = this.props;
      var style = _props3.style;
      var children = _props3.children;
      var layout = _props3.layout;
      var className = _props3.className;
      var width = _props3.width;
      var height = _props3.height;

      var numberAxisName = layout === 'horizontal' ? 'yAxis' : 'xAxis';
      var items = (0, _ReactUtils.findAllByType)(children, _Area2.default);
      var stackGroups = this.getStackGroupsByAxisId(items, numberAxisName + 'Id');

      var xAxisMap = this.getAxisMap('xAxis', items, numberAxisName === 'xAxis' && stackGroups);
      var yAxisMap = this.getAxisMap('yAxis', items, numberAxisName === 'yAxis' && stackGroups);
      var offset = this.getOffset(items, xAxisMap, yAxisMap);

      xAxisMap = this.getFormatAxisMap(xAxisMap, offset, 'xAxis');
      yAxisMap = this.getFormatAxisMap(yAxisMap, offset, 'yAxis');

      return _react2.default.createElement(
        'div',
        { className: (0, _classnames2.default)('recharts-wrapper', className),
          style: _extends({ position: 'relative', cursor: 'default' }, style),
          onMouseEnter: this.handleMouseEnter.bind(this, offset, xAxisMap, yAxisMap),
          onMouseMove: this.handleMouseMove.bind(this, offset, xAxisMap, yAxisMap),
          onMouseLeave: this.handleMouseLeave
        },
        _react2.default.createElement(
          _Surface2.default,
          { width: width, height: height },
          this.renderGrid(xAxisMap, yAxisMap, offset),
          this.renderReferenceLines(xAxisMap, yAxisMap, offset),
          this.renderReferenceDots(xAxisMap, yAxisMap, offset),
          this.renderXAxis(xAxisMap),
          this.renderYAxis(yAxisMap),
          this.renderCursor(xAxisMap, yAxisMap, offset),
          this.renderItems(items, xAxisMap, yAxisMap, offset, stackGroups)
        ),
        this.renderLegend(items),
        this.renderTooltip(items, offset)
      );
    }
  }]);

  return AreaChart;
}(_CartesianChart3.default), _class2.displayName = 'AreaChart', _class2.defaultProps = {
  style: {},
  layout: 'horizontal',
  margin: { top: 5, right: 5, bottom: 5, left: 5 }
}, _temp2)) || _class;

exports.default = AreaChart;